'FMA Script Framework Core Class
'PluginManagerClass
'Interface to the plugin classes

'Interface:
'	-Attributes:
' 	-Self (Let/Get)
'		-PluginManager.Plugin( className ):object ^= PluginManager( className ):object
'	-Methods:
'		-LoadPlugin( className )
'		-UnloadPlugin( className )
'		-IsLoaded( className ):boolean
'		-GetLoadedPluginsArray:array
'		-GetLoadedPluginsLinkedList:LinkedList
'		-IsClassImported( className ):boolean
'		-GetImportedClassesArray:array
'		-GetImportedClassesLinkedList:LinkedList
'		-LoaderString:string

'TODO
'-plugin dependencies? could be a hard job...

Class PluginManagerClass
	
	Private m_Self
	Private importedClasses
	Private loadedPlugins
	
	Sub Class_Initialize()
		Set importedClasses = New Hash
		Set loadedPlugins   = New Hash
	End Sub
	
	Public Property Let Self (s)
		m_Self = s
	End Property
	Public Property Get Self
		Self = m_Self
	End Property
	
	'Return plugin object by given plugin class name
	Public Default Property Get Plugin( className )
		If IsLoaded(className) Then
			Set Plugin = loadedPlugins(className)
		Else
			Set Plugin = Nothing
		End If
	End Property
	
	'Create object from class, append to loaded plugin list
	Sub LoadPlugin( className )
		Debug.DebugMsg "Loading plugin: " & className
		If IsClassImported(className) Then 'Class already imported?
			Dim plugin
			Set plugin = Util.CreateObject(className, Self & "(""" & className & """)")
			Set loadedPlugins(className) = plugin
			'Debug.DebugMsg "Plugin: " & className & " loaded"
		Else
			Debug.DebugMsg "Plugin: " & className & " NOT loaded. Class isn't imported yet!"
		End If
	End Sub
	
	'Kill object, remove from loaded plugin list
	Sub UnloadPlugin( className )
		Debug.DebugMsg "Unloading plugin: " & className
		If IsLoaded(className) Then
			loadedPlugins.DeleteItem className
		End If
	End Sub
	
	'Is this one loaded?
	Function IsLoaded( className )
		IsLoaded = (Not IsEmpty(loadedPlugins(className)))
	End Function
	
	'returns an (unsorted) array with the class names of all loaded plugins
	Function GetLoadedPluginsArray()
		GetLoadedPluginsArray = loadedPlugins.GetKeysArray
	End Function
	
	'returns an (unsorted) linked list with the class names of all loaded plugins
	Function GetLoadedPluginsLinkedList()
		Set GetLoadedPluginsLinkedList = loadedPlugins.GetKeysLinkedList
	End Function
	
	Function IsClassImported( className )
		IsClassImported = (Not IsEmpty(importedClasses(className))) And importedClasses(className)
	End Function
	
	'returns an (unsorted) array with the class names of all imported classes
	Function GetImportedClassesArray()
		GetImportedClassesArray = importedClasses.GetKeysArray
	End Function
	
	'returns an (unsorted) linked list with the class names of imported classes
	Function GetImportedClassesLinkedList()
		Set GetImportedClassesLinkedList = importedClasses.GetKeysLinkedList
	End Function
	
	' Generates a string which will include the pluginclasses and create the plugin objects when Execute()'ed
	Function LoaderString()
		LoaderString = ""
		If Fso.FolderExists(ScriptFolder & "plugins") Then
			Dim pluginFolder, pluginFiles, pluginFile, className, objectName
			Set pluginFolder = Fso.GetFolder(ScriptFolder & "plugins")
			Set pluginFiles = pluginFolder.Files
			
			For Each pluginFile in pluginFiles
				If LCase(Right(pluginFile, 4)) = ".vbs" Then 'Only load .vbs files
					'Determine class and object name
					className = pluginFile
					If InStr(pluginFile.Name, ".") > 0 Then className = Left(pluginFile.Name, InStr(pluginFile.Name, ".") - 1)
					'Include the class file
					'LoaderString = LoaderString & "Debug.DebugMsg ""Executing : " & pluginFile.Path & """" & vbCrLf
					LoaderString = LoaderString & "Execute(ReadFile(""" & pluginFile.Path & """))" & vbCrLf
					importedClasses(className) = True
					'Load the plugin via the plugin manager
					LoaderString = LoaderString & Self & ".LoadPlugin(""" & className & """)" & vbCrLf
				End If
			Next
		Else
			Debug.ErrorMsg Self & ": Folder " & ScriptFolder & "plugin" & " doesn't exist. No plugins loaded!"
		End If
	End Function
	
End Class

