unit uGlobal;

{
*******************************************************************************
* Descriptions: Global
* $Source: /cvsroot/fma/fma/uGlobal.pas,v $
* $Locker:  $
*
* Todo:
*
* Change Log:
* $Log: uGlobal.pas,v $
* Revision 1.7  2004/05/19 18:34:15  z_stoichev
* Build 0.1.0.35c
*
* Revision 1.6  2003/12/04 16:30:43  z_stoichev
* Fixed memory leak,
* Added new function GetTokenCount.
*
* Revision 1.5  2003/11/28 09:38:07  z_stoichev
* Merged with branch-release-1-1 (Fma 0.10.28c)
*
* Revision 1.4.2.2  2003/11/27 12:56:10  z_stoichev
* Add sent message sound.
*
* Revision 1.4.2.1  2003/10/27 07:22:54  z_stoichev
* Build 0.1.0 RC1 Initial Checkin.
*
* Revision 1.4  2003/07/02 12:40:30  crino77
* added call sound
*
* Revision 1.3  2003/01/30 04:15:57  warren00
* Updated with header comments
*
*
*******************************************************************************
}

interface

function GetFirstToken(var str: WideString; delimiter: char = ','): WideString;

function GetToken(str: String; index: Integer): String;
function GetTokenCount(str: String): Integer;

procedure InitRegistry;

implementation

uses Classes, Registry, SysUtils;

function GetFirstToken(var str: WideString; delimiter: char = ','): WideString;
var
  i: integer;
  s: WideString;
//  a: string;
//  p: PChar;
begin
  { Get first token }
  i := Pos(delimiter,str);
  if i = 0 then i := Length(str) + 1;
  s := Copy(str,1,i-1);
  { Update source }
  Delete(str,1,i);
  { Extract quoted value if any 
  a := s;
  p := @a[1];
  a := AnsiExtractQuotedStr(p,'"');
  if a <> '' then Result := a
    else Result := s;
  { widestring method }
  i := Length(s);
  if (i > 1) and (s[1] = '"') and (s[i] = '"') then begin
    Delete(s,i,1);
    Delete(s,1,1);
  end;
  Result := s;
  {}
end;

function GetToken(str: String; index: Integer): String;
var
  sl: TStrings;
begin
  Result := '';
  try
    sl := TStringList.Create;
    try
      sl.DelimitedText := str;
      Result := sl.Strings[index];
    finally
      sl.Free;
    end;
  except
  end;
end;

function GetTokenCount(str: String): Integer;
var
  sl: TStrings;
begin
  Result := 0;
  try
    sl := TStringList.Create;
    try
      sl.DelimitedText := str;
      Result := sl.Count;
    finally
      sl.Free;
    end;
  except
  end;
end;

procedure InitRegistry;
var
  reg: TRegistry;
begin
  reg := TRegistry.Create;

  reg.OpenKey('\AppEvents\EventLabels\FMA_SMSReceived', True);
  reg.WriteString('', 'SMS Received');
  reg.CloseKey;

  reg.OpenKey('\AppEvents\EventLabels\FMA_CallReceived', True);
  reg.WriteString('', 'Call Received');
  reg.CloseKey;

  reg.OpenKey('\AppEvents\EventLabels\FMA_MEConnected', True);
  reg.WriteString('', 'Connected to Mobile Equipment');
  reg.CloseKey;

  reg.OpenKey('\AppEvents\EventLabels\FMA_MEDisconnected', True);
  reg.WriteString('', 'Disconnected from Mobile Equipment');
  reg.CloseKey;

  reg.OpenKey('\AppEvents\Schemes\Apps\MobileAgent', True);
  reg.WriteString('', 'floAt''s Mobile Agent');
  reg.CloseKey;

  reg.CreateKey('\AppEvents\Schemes\Apps\MobileAgent\FMA_CallReceived');
  reg.CreateKey('\AppEvents\Schemes\Apps\MobileAgent\FMA_SMSReceived');
  reg.CreateKey('\AppEvents\Schemes\Apps\MobileAgent\FMA_SMSSent');
  reg.CreateKey('\AppEvents\Schemes\Apps\MobileAgent\FMA_MEConnected');
  reg.CreateKey('\AppEvents\Schemes\Apps\MobileAgent\FMA_MEDisconnected');

  reg.Free;
end;

end.
