unit uNewMessage;

{
*******************************************************************************
* Descriptions: Incoming Message Pop-up Implementation
* $Source: /cvsroot/fma/fma/uNewMessage.pas,v $
* $Locker:  $
*
* Todo:
*
* Change Log:
* $Log: uNewMessage.pas,v $
* Revision 1.9  2004/04/01 15:06:21  z_stoichev
* Mark as read on reply/forward
* popup menu moved
*
* Revision 1.8  2004/01/12 15:36:05  z_stoichev
* Fixed too small width issue.
*
* Revision 1.7  2003/11/28 09:38:07  z_stoichev
* Merged with branch-release-1-1 (Fma 0.10.28c)
*
* Revision 1.6.2.4  2003/11/21 14:00:23  z_stoichev
* Message fade out stopped on mouse over text or when
* window is activated.
* Actions as delete, reply, etc. moved into Popup menu.
* Added Add contact action for unknown senders.
* Window shows inactivated in order do not steal focus.
*
* Revision 1.6.2.3  2003/11/11 18:09:22  z_stoichev
* Add background.
* Auto form resize.
* Add some buttons.
*
* Revision 1.6.2.2  2003/11/10 14:03:09  z_stoichev
* RC3
*
* Revision 1.6.2.1  2003/10/27 07:22:54  z_stoichev
* Build 0.1.0 RC1 Initial Checkin.
*
* Revision 1.6  2003/10/14 09:23:47  z_stoichev
* Unit SentSMS renamed to uComposeSMS.
*
* Revision 1.5  2003/10/14 09:14:44  z_stoichev
* Method Create renamed to CreateMsg in order
* to avoid hiding of virtual inherited method.
*
* Revision 1.4  2003/01/30 04:15:57  warren00
* Updated with header comments
*
*
*******************************************************************************
}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Placemnt, ExtCtrls, TntStdCtrls, jpeg, Menus;

type
  TfrmNewMessage = class(TForm)
    FormPlacement1: TFormPlacement;
    Timer1: TTimer;
    lbText: TTntLabel;
    lbAlpha: TTntLabel;
    Image1: TImage;
    OkButton: TButton;
    ActionButton: TTntButton;
    PopupMenu1: TPopupMenu;
    Reply1: TMenuItem;
    Forward1: TMenuItem;
    N1: TMenuItem;
    Delete1: TMenuItem;
    N2: TMenuItem;
    AddContact1: TMenuItem;
    procedure Timer1Timer(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure Reply1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure OkButtonClick(Sender: TObject);
    procedure ActionButtonClick(Sender: TObject);
    procedure Forward1Click(Sender: TObject);
    procedure PopupMenu1Popup(Sender: TObject);
    procedure Delete1Click(Sender: TObject);
    procedure OnMouseEnter(Sender: TObject);
    procedure OnMouseLeave(Sender: TObject);
    procedure FormDeactivate(Sender: TObject);
    procedure AddContact1Click(Sender: TObject);
  private
    { Private declarations }
    FAlphaCount: integer;
    FPrevAlpha: boolean;
    procedure MarkMsgAsRead;
  public
    { Public declarations }
    msgLocation,msgPdu: String;
    msgIndex: Integer;
    constructor CreateMsg(Sender, text: WideString; AlphaBlend: Integer);
  end;

var
  frmNewMessage: TfrmNewMessage;

implementation

uses Unit1, uComposeSMS;

{$R *.dfm}

const
  DontFadeSeconds = 5;

constructor TfrmNewMessage.CreateMsg(Sender, text: WideString; AlphaBlend: Integer);
var
  wide: integer;
begin
  inherited Create(nil);
  msgLocation := '';
  msgPdu := '';
  msgIndex := -1;
  { Prepare fade out }
  AlphaBlendValue := AlphaBlend;
  FAlphaCount := AlphaBlend + DontFadeSeconds*10; // dont fade immediately
  { Resize form to fix message }
  lbAlpha.Alignment := taLeftJustify;
  lbAlpha.AutoSize := True;
  lbAlpha.Anchors := [akLeft,akTop];
  lbAlpha.Caption := Sender;
  lbText.Anchors := [akLeft,akTop];
  lbText.Caption := '';
  { Restore form position }
  FormPlacement1.RestoreFormPlacement;
  //Show;
  lbText.WordWrap := False;
  lbText.AutoSize := True;
  lbText.Caption := text;
  lbText.WordWrap := True;
  { Resize form to fit message }
  wide := lbText.Width;
  if lbAlpha.Width > wide then wide := lbAlpha.Width;
  if wide > (Constraints.MaxWidth-16) then begin
    Width := Constraints.MaxWidth;
    lbText.AutoSize := False;
    lbText.Width := Constraints.MaxWidth-16;
    lbText.AutoSize := True;
    ClientHeight := lbText.Height + 60;
  end
  else begin
    if wide > (Constraints.MinWidth-16) then
      Width := wide + 16
    else
      Width := Constraints.MinWidth;
    Height := Constraints.MinHeight;
  end;
  lbText.Anchors := [akLeft,akTop,akRight,akBottom];
  lbText.AutoSize := False;
  lbAlpha.AutoSize := False;
  lbAlpha.Width := Width-16;
  lbAlpha.Alignment := taCenter;
  lbAlpha.Anchors := [akLeft,akTop,akRight];
  { Fix buttons top }
  OkButton.Top := ClientHeight - 25;
  ActionButton.Top := OkButton.Top;
  { Show window but not activate it }
  ShowWindow(Handle,SW_SHOWNOACTIVATE);
  ShowWindow(OkButton.Handle,SW_SHOWNOACTIVATE);
  ShowWindow(ActionButton.Handle,SW_SHOWNOACTIVATE);
  { Run fade out }
  Timer1.Interval := 100;
  Timer1.Enabled := True;
end;

procedure TfrmNewMessage.Timer1Timer(Sender: TObject);
begin
  FAlphaCount := FAlphaCount - 1;
  if FAlphaCount < AlphaBlendValue then begin
    AlphaBlendValue := AlphaBlendValue - 1;
    if AlphaBlendValue < 10 then
      OkButton.Click;
  end;
end;

procedure TfrmNewMessage.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  Action := caFree;
end;

procedure TfrmNewMessage.FormCreate(Sender: TObject);
begin
  Image1.Picture.Assign(Form1.BackgrImage.Picture);
end;

procedure TfrmNewMessage.OkButtonClick(Sender: TObject);
begin
  Timer1.Enabled := False;
  Close;
end;

procedure TfrmNewMessage.ActionButtonClick(Sender: TObject);
var
  p: TPoint;
begin
  with (Sender as TButton) do
    p := ClientToScreen(Point(0,Height));
  PopupMenu1.Popup(p.X,p.Y);  
end;

procedure TfrmNewMessage.PopupMenu1Popup(Sender: TObject);
begin
  Delete1.Enabled := msgIndex <> -1;
  AddContact1.Enabled := Pos('[',lbAlpha.Caption) = 0; // no name included ('name [number]')
end;

procedure TfrmNewMessage.Reply1Click(Sender: TObject);
begin
  MarkMsgAsRead;
  NewMsgForm.Clear;
  Form1.ActionSMSNewMsg.Execute;
  NewMsgForm.AddRecipient(lbAlpha.Caption);
end;

procedure TfrmNewMessage.Forward1Click(Sender: TObject);
begin
  MarkMsgAsRead;
  NewMsgForm.Clear;
  Form1.ActionSMSNewMsg.Execute;
  NewMsgForm.Memo.Text := lbText.Caption;
  NewMsgForm.Memo.SelStart := Length(lbText.Caption);
end;

procedure TfrmNewMessage.Delete1Click(Sender: TObject);
begin
  if Form1.DeleteSMS(msgIndex,msgLocation,msgPdu) then begin
    OkButton.Click;
  end;
end;

procedure TfrmNewMessage.AddContact1Click(Sender: TObject);
begin
  { now lbAlpha.Caption comtains the number only }
  Form1.frmSyncPhonebook.DoEdit(True,lbAlpha.Caption);
end;

procedure TfrmNewMessage.OnMouseEnter(Sender: TObject);
begin
  FPrevAlpha := AlphaBlend;
  AlphaBlend := False;
  Timer1.Enabled := False;
end;

procedure TfrmNewMessage.OnMouseLeave(Sender: TObject);
begin
  if FPrevAlpha then
    FormDeactivate(nil);
end;

procedure TfrmNewMessage.FormDeactivate(Sender: TObject);
begin
  { Was fade out started when we enter ? }
  if FAlphaCount = AlphaBlendValue then begin
    { Yes, fade out faster now }
    Timer1.Interval := 5;
    FAlphaCount := 250;
    AlphaBlendValue := 250;
  end; { No, just continue }
  AlphaBlend := True;
  Timer1.Enabled := True;
end;

procedure TfrmNewMessage.MarkMsgAsRead;
begin
  { Mark as read }
  if Form1.FTextMessageOptions.MoveToArchive then
    Form1.UpdateNewMessagesCounter(Form1.FNodeMsgArchive,msgPdu)
  else
    Form1.UpdateNewMessagesCounter(Form1.FNodeMsgInbox,msgPdu);
end;

end.
